@@ -36,9 +36,10 @@ class PyDriveBackend(duplicity.backend.Backend):
             from pydrive.auth import GoogleAuth
             from pydrive.drive import GoogleDrive
             from pydrive.files import FileNotUploadedError
-        except ImportError:
-            raise BackendException('PyDrive backend requires PyDrive installation'
-                                   'Please read the manpage to fix.')
+        except ImportError as e:
+            raise BackendException("""\
+PyDrive backend requires PyDrive installation.  Please read the manpage for setup details.
+Exception: %s""" % str(e))
 
         # let user get by with old client while he can
         try:
@@ -66,6 +67,17 @@ class PyDriveBackend(duplicity.backend.Backend):
         elif 'GOOGLE_DRIVE_SETTINGS' in os.environ:
             gauth = GoogleAuth(settings_file=os.environ['GOOGLE_DRIVE_SETTINGS'])
             gauth.CommandLineAuth()
+        elif ('GOOGLE_SECRETS_FILE' in os.environ and 'GOOGLE_CREDENTIALS_FILE' in os.environ):
+            gauth = GoogleAuth()
+            gauth.LoadClientConfigFile(os.environ['GOOGLE_SECRETS_FILE'])
+            gauth.LoadCredentialsFile(os.environ['GOOGLE_CREDENTIALS_FILE'])
+            if gauth.credentials is None:
+                gauth.CommandLineAuth()
+            elif gauth.access_token_expired:
+                gauth.Refresh()
+            else:
+                gauth.Authorize()
+            gauth.SaveCredentialsFile(os.environ['GOOGLE_CREDENTIALS_FILE'])
         else:
             raise BackendException(
                 'GOOGLE_DRIVE_ACCOUNT_KEY or GOOGLE_DRIVE_SETTINGS environment '
@@ -199,11 +211,7 @@ class PyDriveBackend(duplicity.backend.Backend):
         if isinstance(error, FileNotUploadedError):
             return log.ErrorCode.backend_not_found
         elif isinstance(error, ApiRequestError):
-            http_status = error.args[0].resp.status
-            if http_status == 404:
-                return log.ErrorCode.backend_not_found
-            elif http_status == 403:
-                return log.ErrorCode.backend_permission_denied
+            return log.ErrorCode.backend_permission_denied
         return log.ErrorCode.backend_error
 
 duplicity.backend.register_backend('pydrive', PyDriveBackend)
