@@ -58,31 +58,26 @@ class VerifiedHTTPSConnection(httplib.HTTPSConnection):
                 import socket
                 import ssl
             except ImportError:
-                raise FatalBackendException("Missing socket or ssl libraries.")
+                raise FatalBackendException(_("Missing socket or ssl python modules."))
 
             httplib.HTTPSConnection.__init__(self, *args, **kwargs)
 
             self.cacert_file = globals.ssl_cacert_file
-            cacert_candidates = ["~/.duplicity/cacert.pem",
-                                 "~/duplicity_cacert.pem",
-                                 "/etc/duplicity/cacert.pem"]
-            #
+            self.cacert_candidates = ["~/.duplicity/cacert.pem",
+                                      "~/duplicity_cacert.pem",
+                                      "/etc/duplicity/cacert.pem"]
+            # if no cacert file was given search default locations
             if not self.cacert_file:
-                for path in cacert_candidates:
+                for path in self.cacert_candidates:
                     path = os.path.expanduser(path)
                     if (os.path.isfile(path)):
                         self.cacert_file = path
                         break
-            # still no cacert file, inform user
-            if not self.cacert_file:
-                raise FatalBackendException("""\
-For certificate verification a cacert database file is needed in one of these locations: %s
-Hints:
-  Consult the man page, chapter 'SSL Certificate Verification'.
-  Consider using the options --ssl-cacert-file, --ssl-no-check-certificate .""" % ", ".join(cacert_candidates))
+
             # check if file is accessible (libssl errors are not very detailed)
-            if not os.access(self.cacert_file, os.R_OK):
-                raise FatalBackendException("Cacert database file '%s' is not readable." % self.cacert_file)
+            if self.cacert_file and not os.access(self.cacert_file, os.R_OK):
+                raise FatalBackendException(_("Cacert database file '%s' is not readable.") %
+                                            self.cacert_file)
 
         def connect(self):
             # create new socket
@@ -92,18 +87,41 @@ Hints:
                 self.sock = sock
                 self.tunnel()
 
-            # wrap the socket in ssl using verification
-            self.sock = ssl.wrap_socket(sock,
-                                        cert_reqs=ssl.CERT_REQUIRED,
-                                        ca_certs=self.cacert_file,
-                                        )
+            # python 2.7.9+ supports default system certs now
+            if "create_default_context" in dir(ssl):
+                context = ssl.create_default_context(ssl.Purpose.SERVER_AUTH,
+                                                     cafile=self.cacert_file,
+                                                     capath=globals.ssl_cacert_path)
+                self.sock = context.wrap_socket(sock, server_hostname=self.host)
+            # the legacy way needing a cert file
+            else:
+                if globals.ssl_cacert_path:
+                    raise FatalBackendException(
+                        _("Option '--ssl-cacert-path' is not supported "
+                          "with python 2.7.8 and below."))
+
+                if not self.cacert_file:
+                    raise FatalBackendException(_("""\
+For certificate verification with python 2.7.8 or earlier a cacert database
+file is needed in one of these locations: %s
+Hints:
+  Consult the man page, chapter 'SSL Certificate Verification'.
+  Consider using the options --ssl-cacert-file, --ssl-no-check-certificate .""") %
+                                                ", ".join(self.cacert_candidates))
+
+                # wrap the socket in ssl using verification
+                self.sock = ssl.wrap_socket(sock,
+                                            cert_reqs=ssl.CERT_REQUIRED,
+                                            ca_certs=self.cacert_file,
+                                            )
 
         def request(self, *args, **kwargs):  # pylint: disable=method-hidden
             try:
                 return httplib.HTTPSConnection.request(self, *args, **kwargs)
             except ssl.SSLError as e:
                 # encapsulate ssl errors
-                raise BackendException("SSL failed: %s" % util.uexc(e), log.ErrorCode.backend_error)
+                raise BackendException("SSL failed: %s" % util.uexc(e),
+                                       log.ErrorCode.backend_error)
 
 
 class WebDAVBackend(duplicity.backend.Backend):
@@ -113,13 +131,9 @@ class WebDAVBackend(duplicity.backend.Backend):
     """
 
     """
-    for better compatibility we send an empty listbody as described in
-    http://www.ietf.org/rfc/rfc4918.txt
-    "  A client may choose not to submit a request body.  An empty PROPFIND
-       request body MUST be treated as if it were an 'allprop' request.  "
-    it was retired because e.g. box.net didn't support <D:allprop/>
+    Request just the names.
     """
-    listbody = ""
+    listbody = '<?xml version="1.0"?><D:propfind xmlns:D="DAV:"><D:prop><D:resourcetype/></D:prop></D:propfind>'
 
     """Connect to remote store using WebDAV Protocol"""
     def __init__(self, parsed_url):
@@ -133,9 +147,10 @@ class WebDAVBackend(duplicity.backend.Backend):
         self.password = self.get_password()
         self.directory = self.sanitize_path(parsed_url.path)
 
-        log.Info("Using WebDAV protocol %s" % (globals.webdav_proto,))
-        log.Info("Using WebDAV host %s port %s" % (parsed_url.hostname, parsed_url.port))
-        log.Info("Using WebDAV directory %s" % (self.directory,))
+        log.Info(_("Using WebDAV protocol %s") % (globals.webdav_proto,))
+        log.Info(_("Using WebDAV host %s port %s") % (parsed_url.hostname,
+                                                      parsed_url.port))
+        log.Info(_("Using WebDAV directory %s") % (self.directory,))
 
         self.conn = None
 
@@ -166,7 +181,7 @@ class WebDAVBackend(duplicity.backend.Backend):
                 and self.conn.host == self.parsed_url.hostname:
             return
 
-        log.Info("WebDAV create connection on '%s'" % (self.parsed_url.hostname))
+        log.Info(_("WebDAV create connection on '%s'") % (self.parsed_url.hostname))
         self._close()
         # http schemes needed for redirect urls from servers
         if self.parsed_url.scheme in ['webdav', 'http']:
@@ -177,7 +192,7 @@ class WebDAVBackend(duplicity.backend.Backend):
             else:
                 self.conn = VerifiedHTTPSConnection(self.parsed_url.hostname, self.parsed_url.port)
         else:
-            raise FatalBackendException("WebDAV Unknown URI scheme: %s" % (self.parsed_url.scheme))
+            raise FatalBackendException(_("WebDAV Unknown URI scheme: %s") % (self.parsed_url.scheme))
 
     def _close(self):
         if self.conn:
@@ -196,34 +211,34 @@ class WebDAVBackend(duplicity.backend.Backend):
         if self.digest_challenge is not None:
             self.headers['Authorization'] = self.get_digest_authorization(path)
 
-        log.Info("WebDAV %s %s request with headers: %s " % (method, quoted_path, self.headers))
-        log.Info("WebDAV data length: %s " % len(str(data)))
+        log.Info(_("WebDAV %s %s request with headers: %s ") % (method, quoted_path, self.headers))
+        log.Info(_("WebDAV data length: %s ") % len(str(data)))
         self.conn.request(method, quoted_path, data, self.headers)
         response = self.conn.getresponse()
-        log.Info("WebDAV response status %s with reason '%s'." % (response.status, response.reason))
+        log.Info(_("WebDAV response status %s with reason '%s'.") % (response.status, response.reason))
         # resolve redirects and reset url on listing requests (they usually come before everything else)
         if response.status in [301, 302] and method == 'PROPFIND':
             redirect_url = response.getheader('location', None)
             response.close()
             if redirect_url:
-                log.Notice("WebDAV redirect to: %s " % urllib.unquote(redirect_url))
+                log.Notice(_("WebDAV redirect to: %s ") % urllib.unquote(redirect_url))
                 if redirected > 10:
-                    raise FatalBackendException("WebDAV redirected 10 times. Giving up.")
+                    raise FatalBackendException(_("WebDAV redirected 10 times. Giving up."))
                 self.parsed_url = duplicity.backend.ParsedUrl(redirect_url)
                 self.directory = self.sanitize_path(self.parsed_url.path)
                 return self.request(method, self.directory, data, redirected + 1)
             else:
-                raise FatalBackendException("WebDAV missing location header in redirect response.")
+                raise FatalBackendException(_("WebDAV missing location header in redirect response."))
         elif response.status == 401:
             response.read()
             response.close()
             self.headers['Authorization'] = self.get_authorization(response, quoted_path)
-            log.Info("WebDAV retry request with authentification headers.")
-            log.Info("WebDAV %s %s request2 with headers: %s " % (method, quoted_path, self.headers))
-            log.Info("WebDAV data length: %s " % len(str(data)))
+            log.Info(_("WebDAV retry request with authentification headers."))
+            log.Info(_("WebDAV %s %s request2 with headers: %s ") % (method, quoted_path, self.headers))
+            log.Info(_("WebDAV data length: %s ") % len(str(data)))
             self.conn.request(method, quoted_path, data, self.headers)
             response = self.conn.getresponse()
-            log.Info("WebDAV response2 status %s with reason '%s'." % (response.status, response.reason))
+            log.Info(_("WebDAV response2 status %s with reason '%s'.") % (response.status, response.reason))
 
         return response
 
@@ -236,7 +251,18 @@ class WebDAVBackend(duplicity.backend.Backend):
             token, challenge = auth_hdr.split(' ', 1)
         except ValueError:
             return None
-        if token.lower() == 'basic':
+        if token.split(',')[0].lower() == 'negotiate':
+            try:
+                return self.get_kerberos_authorization()
+            except ImportError:
+                log.Warn(_("python-kerberos needed to use kerberos \
+                          authorization, falling back to basic auth."))
+                return self.get_basic_authorization()
+            except Exception as e:
+                log.Warn(_("Kerberos authorization failed: %s.\
+                          Falling back to basic auth.") % e)
+                return self.get_basic_authorization()
+        elif token.lower() == 'basic':
             return self.get_basic_authorization()
         else:
             self.digest_challenge = self.parse_digest_challenge(challenge)
@@ -245,6 +271,13 @@ class WebDAVBackend(duplicity.backend.Backend):
     def parse_digest_challenge(self, challenge_string):
         return urllib2.parse_keqv_list(urllib2.parse_http_list(challenge_string))
 
+    def get_kerberos_authorization(self):
+        import kerberos  # pylint: disable=import-error
+        _, ctx = kerberos.authGSSClientInit("HTTP@%s" % self.conn.host)
+        kerberos.authGSSClientStep(ctx, "")
+        tgt = kerberos.authGSSClientResponse(ctx)
+        return 'Negotiate %s' % tgt
+
     def get_basic_authorization(self):
         """
         Returns the basic auth header
@@ -268,7 +301,8 @@ class WebDAVBackend(duplicity.backend.Backend):
         hostname = u.port and "%s:%s" % (u.hostname, u.port) or u.hostname
         dummy_url = "%s://%s%s" % (scheme, hostname, path)
         dummy_req = CustomMethodRequest(self.conn._method, dummy_url)
-        auth_string = self.digest_auth_handler.get_authorization(dummy_req, self.digest_challenge)
+        auth_string = self.digest_auth_handler.get_authorization(dummy_req,
+                                                                 self.digest_challenge)
         return 'Digest %s' % auth_string
 
     def _list(self):
@@ -323,11 +357,12 @@ class WebDAVBackend(duplicity.backend.Backend):
             log.Info("Checking existence dir %s: %d" % (d, response.status))
 
             if response.status == 404:
-                log.Info("Creating missing directory %s" % d)
+                log.Info(_("Creating missing directory %s") % d)
 
                 res = self.request("MKCOL", d)
                 if res.status != 201:
-                    raise BackendException("WebDAV MKCOL %s failed: %s %s" % (d, res.status, res.reason))
+                    raise BackendException(_("WebDAV MKCOL %s failed: %s %s") %
+                                           (d, res.status, res.reason))
 
     def taste_href(self, href):
         """
@@ -339,8 +374,8 @@ class WebDAVBackend(duplicity.backend.Backend):
         raw_filename = self.getText(href.childNodes).strip()
         parsed_url = urlparse.urlparse(urllib.unquote(raw_filename))
         filename = parsed_url.path
-        log.Debug("webdav path decoding and translation: "
-                  "%s -> %s" % (raw_filename, filename))
+        log.Debug(_("WebDAV path decoding and translation: "
+                  "%s -> %s") % (raw_filename, filename))
 
         # at least one WebDAV server returns files in the form
         # of full URL:s. this may or may not be
@@ -376,14 +411,16 @@ class WebDAVBackend(duplicity.backend.Backend):
                 # data=response.read()
                 target_file.write(response.read())
                 # import hashlib
-                # log.Info("WebDAV GOT %s bytes with md5=%s" % (len(data),hashlib.md5(data).hexdigest()) )
+                # log.Info("WebDAV GOT %s bytes with md5=%s" %
+                # (len(data),hashlib.md5(data).hexdigest()) )
                 assert not target_file.close()
                 response.close()
             else:
                 status = response.status
                 reason = response.reason
                 response.close()
-                raise BackendException("Bad status code %s reason %s." % (status, reason))
+                raise BackendException(_("WebDAV GET Bad status code %s reason %s.") %
+                                       (status, reason))
         except Exception as e:
             raise e
         finally:
@@ -404,7 +441,8 @@ class WebDAVBackend(duplicity.backend.Backend):
                 status = response.status
                 reason = response.reason
                 response.close()
-                raise BackendException("Bad status code %s reason %s." % (status, reason))
+                raise BackendException(_("WebDAV PUT Bad status code %s reason %s.") %
+                                       (status, reason))
         except Exception as e:
             raise e
         finally:
@@ -423,7 +461,8 @@ class WebDAVBackend(duplicity.backend.Backend):
                 status = response.status
                 reason = response.reason
                 response.close()
-                raise BackendException("Bad status code %s reason %s." % (status, reason))
+                raise BackendException(_("WebDAV DEL Bad status code %s reason %s.") %
+                                       (status, reason))
         except Exception as e:
             raise e
         finally:
