@@ -20,7 +20,8 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-import sys, os
+import sys
+import os
 from distutils.core import setup, Extension
 
 assert len(sys.argv) == 1
