@@ -46,6 +46,7 @@ import sys
 
 from duplicity import errors
 
+
 def threading_supported():
     """
     Returns whether threading is supported on the system we are
@@ -53,6 +54,7 @@ def threading_supported():
     """
     return _threading_supported
 
+
 def require_threading(reason=None):
     """
     Assert that threading is required for operation to continue. Raise
@@ -65,9 +67,10 @@ def require_threading(reason=None):
     if not threading_supported():
         if reason is None:
             reason = "(no reason given)"
-        raise errors.NotSupported("threading was needed because [%s], but "\
-                                      "is not supported by the python "\
-                                      "interpreter" % (reason,))
+        raise errors.NotSupported("threading was needed because [%s], but "
+                                  "is not supported by the python "
+                                  "interpreter" % (reason,))
+
 
 def thread_module():
     """
@@ -76,6 +79,7 @@ def thread_module():
     """
     return thread
 
+
 def threading_module():
     """
     Returns the threading module, or dummy_thread if threading is not
@@ -83,6 +87,7 @@ def threading_module():
     """
     return threading
 
+
 def with_lock(lock, fn):
     """
     Call fn with lock acquired. Guarantee that lock is released upon
@@ -101,6 +106,7 @@ def with_lock(lock, fn):
     finally:
         lock.release()
 
+
 def interruptably_wait(cv, waitFor):
     """
     cv   - The threading.Condition instance to wait on
@@ -152,6 +158,7 @@ def interruptably_wait(cv, waitFor):
     while not waitFor():
         cv.wait(0.1)
 
+
 def async_split(fn):
     """
     Splits the act of calling the given function into one front-end
@@ -177,12 +184,11 @@ def async_split(fn):
     # not care about hash lookup overhead since this is intended to be
     # used for significant amounts of work.
 
-
     cv = threading.Condition()  # @UndefinedVariable
-    state = { 'done': False,
-              'error': None,
-              'trace': None,
-              'value': None }
+    state = {'done': False,
+             'error': None,
+             'trace': None,
+             'value': None}
 
     def waiter():
         cv.acquire()
@@ -219,6 +225,7 @@ def async_split(fn):
 
     return (waiter, caller)
 
+
 class Value:
     """
     A thread-safe container of a reference to an object (but not the
