@@ -24,6 +24,7 @@ from future_builtins import filter, map
 import re  # @UnusedImport
 import types
 import os
+import sys
 import tempfile
 
 from duplicity import tarfile  # @UnusedImport
@@ -491,6 +492,10 @@ def patch_seq2ropath(patch_seq):
             """
             librsync insists on a real file object, which we create manually
             by using the duplicity.tempdir to tell us where.
+
+            See https://bugs.launchpad.net/duplicity/+bug/670891 for discussion
+            of os.tmpfile() vs tempfile.TemporaryFile() w.r.t. Windows / Posix,
+            which is worked around in librsync.PatchedFile() now.
             """
             tempfp = tempfile.TemporaryFile(dir=tempdir.default().dir())
             util.copyfileobj(current_file, tempfp)
